﻿using System;
using System.Collections.Generic;
using System.CommandLine;
using System.CommandLine.Invocation;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using UpToolLib;
using UpToolLib.DataStructures;
using UpToolLib.v1;
using UpToolLib.v1.Tool;
using UpToolLib.v2;

namespace UpToolCLI
{
    public static class PackageManagement
    {
        public static void RegisterCommands(RootCommand rootCommand)
        {
            Command install = new("install", "Install a package")
            {
                new Argument<string>("identifier", "Something to identify the app or the file name"),
                new Option<bool>(new[] {"--force", "-f"}, "Overwrites older files")
            };
            install.Handler = CommandHandler.Create<string, bool>(Install);
            rootCommand.AddCommand(install);

            Command upgrade = new("upgrade", "Upgrade a package")
            {
                new Argument<string>("identifier", "Something to identify the app"),
                new Option<bool>(new[] {"--force", "-f"}, "Overwrites older files")
            };
            upgrade.Handler = CommandHandler.Create<string, bool>(Upgrade);
            rootCommand.AddCommand(upgrade);

            Command reinstall = new("reinstall", "Reinstall a package")
            {
                new Argument<string>("identifier", "Something to identify the app"),
                new Option<bool>(new[] {"--force", "-f"}, "Overwrites older files")
            };
            reinstall.Handler = CommandHandler.Create<string, bool>(Reinstall);
            rootCommand.AddCommand(reinstall);

            Command remove = new("remove", "Remove a package")
            {
                new Argument<string>("identifier", "Something to identify the app")
            };
            remove.Handler = CommandHandler.Create<string>(Remove);
            rootCommand.AddCommand(remove);

            Command purge = new("purge", "Completely remove a package")
            {
                new Argument<string>("identifier", "Something to identify the app")
            };
            purge.Handler = CommandHandler.Create<string>(Purge);
            rootCommand.AddCommand(purge);

            rootCommand.AddCommand(new Command("dist-upgrade", "Upgrades all packages")
            {
                Handler = CommandHandler.Create(DistUpgrade)
            });
        }

        private static void Install(string identifier, bool force)
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            App[] apps = Program.Lib.V1.AppExtras.FindApps(identifier);
            if (apps.Length == 0)
            {
                if (File.Exists(identifier))
                {
                    Console.WriteLine("Name:");
                    string name = Console.ReadLine();
                    Program.Lib.V1.Installer.InstallZip(identifier, Program.Lib.V2.AppFactory.CreateApp(name, "Locally installed package, removal only",
                        UpToolLibV1.MinimumVer, "", true, "",
                        Guid.NewGuid(), Color.Red, "", false, ""), force);
                    Console.WriteLine($"Successfully installed \"{name}\"");
                }
                else
                {
                    Console.WriteLine("Package not found.");
                    Console.WriteLine(identifier);
                }
            }
            else
            {
                App tmp = apps.First();
                if (tmp.Status.Contains(Status.Installed))
                    Console.WriteLine("Package is already installed");
                else
                {
                    Console.WriteLine($"Installing {tmp.Name}");
                    Program.Lib.V1.Installer.Install(tmp, true);
                }
            }
            Console.WriteLine("Done!");
        }

        private static void Upgrade(string identifier, bool force)
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            App[] apps = Program.Lib.V1.AppExtras.FindApps(identifier);
            if (apps.Length == 0)
                Console.WriteLine("Package not found.");
            else
            {
                App tmp = apps.First();
                if (tmp.Status.Contains(Status.Updatable))
                {
                    Console.WriteLine($"Upgrading {tmp.Name}");
                    Program.Lib.V1.AppExtras.Update(tmp, force);
                }
                else
                    Console.WriteLine("Package is up-to-date");
            }
            Console.WriteLine("Done!");
        }

        private static void Reinstall(string identifier, bool force)
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            App[] apps = Program.Lib.V1.AppExtras.FindApps(identifier);
            if (apps.Length == 0)
                Console.WriteLine("Package not found.");
            else
            {
                App tmp = apps.First();
                Console.WriteLine($"Reinstalling {tmp.Name}");
                Program.Lib.V1.AppExtras.Update(tmp, force);
            }
            Console.WriteLine("Done!");
        }

        private static void Remove(string identifier)
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            App[] apps = Program.Lib.V1.AppExtras.FindApps(identifier);
            if (apps.Length == 0)
                Console.WriteLine("Package not found.");
            else
            {
                App tmp = apps.First();
                if (tmp.Status.Contains(Status.Installed))
                {
                    Console.WriteLine($"Removing {tmp.Name}");
                    Program.Lib.V1.AppExtras.Remove(tmp, false);
                }
                else
                    Console.WriteLine("Package is not installed");
            }
            Console.WriteLine("Done!");
        }

        private static void Purge(string identifier)
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            App[] apps = Program.Lib.V1.AppExtras.FindApps(identifier);
            if (apps.Length == 0)
                Console.WriteLine("Package not found.");
            else
            {
                App tmp = apps.First();
                if (tmp.Status.Contains(Status.Installed))
                {
                    Console.WriteLine($"Purging {tmp.Name}");
                    Program.Lib.V1.AppExtras.Remove(tmp, true);
                }
                else
                    Console.WriteLine("Package is not installed");
            }
            Console.WriteLine("Done!");
        }

        private static void DistUpgrade()
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            foreach (KeyValuePair<Guid, App> app in Program.Lib.V1.Apps.Where(s =>
                s.Value.Status.Contains(Status.Updatable)))
            {
                Console.WriteLine($"Updating {app.Value.Name}");
                Program.Lib.V1.AppExtras.Update(app.Value, false);
            }
            if (Assembly.GetExecutingAssembly().GetName().Version < Program.Lib.V2.UpdateChecker.Check().OnlineVersion)
            {
                Console.WriteLine("Updating self");
                Other.UpgradeSelf(false);
            }
            Console.WriteLine("Done!");
        }
    }
}