﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using UpToolLib.DataStructures;
using UpToolLib.v1;
using UpToolLib.v1.Tool;

namespace UpToolLib.v2
{
    public class RepoManagement
    {
        private readonly IExternalFunctionality _platform;
        private readonly XmlTool _xmlTool;
        private readonly PathTool _pathTool;
        private readonly AppFactory _appFactory;
        private readonly IDictionary<Guid, App> _apps;

        public RepoManagement(IExternalFunctionality platform, XmlTool xmlTool, PathTool pathTool, AppFactory appFactory, IDictionary<Guid, App> apps)
        {
            _platform = platform;
            _xmlTool = xmlTool;
            _pathTool = pathTool;
            _appFactory = appFactory;
            _apps = apps;
        }

        public void Save(IEnumerable<Repo> repos)
        {
            XDocument doc = XDocument.Load(_pathTool.InfoXml);
            XElement reposEl = doc.Element("meta").Element("Repos");
            reposEl.RemoveNodes();
            foreach (Repo repo in repos)
                reposEl.Add(new XElement("Repo", new XElement("Name", repo.Name), new XElement("Link", repo.Url)));
            doc.Save(_pathTool.InfoXml);
        }

        public IEnumerable<Repo> GetRepos()
        {
            _xmlTool.FixXml();
            XDocument doc = XDocument.Load(_pathTool.InfoXml);
            XElement reposEl = doc.Element("meta").Element("Repos");
            List<Repo> repos = new();
            foreach (XElement repo in reposEl.Elements("Repo"))
            {
                repos.Add(new Repo
                {
                    Name = repo.Element("Name").Value,
                    Url = repo.Element("Link").Value
                });
            }
            return repos.ToArray();
        }

        public void FetchRepos()
        {
            _xmlTool.FixXml();
            XElement meta = XDocument.Load(_pathTool.InfoXml).Element("meta");
            List<XElement> tmpAppsList = new();
            List<string> repArr = meta.Element("Repos").Elements("Repo").Select(s => s.Element("Link").Value).Distinct()
                .ToList();
            int i = 0;
            while (i < repArr.Count)
            {
                string status = "Initializing";
#if !DEBUG
                try
                {
#endif
                    _platform.Log($"[{i + 1}] Loading {repArr[i]}");
                    status = $"Loading {repArr[i]}";
                    XDocument repo = XDocument.Load(new Uri(repArr[i]).AbsoluteUri);
                    status = $"Extracting repolinks";
                    repArr.AddRange(repo.Element("repo").Elements("repolink").Select(s => s.Value)
                        .Where(s => !repArr.Contains(s)));
                    status = $"Extracting apps";
                    XElement[] tmpApparray = repo.Element("repo").Elements("app").Where(app =>
                            !tmpAppsList.Any(a => a.Element("ID").Value == app.Element("ID").Value) ||
                            !tmpAppsList
                                .Where(a => a.Element("ID").Value == app.Element("ID").Value).Any(a =>
                                    GetVer(a.Element("Version")) >= GetVer(app.Element("Version")))).ToArray()
                        .Concat(repo.Element("repo").Elements("applink")
                            .Select(s =>
                            {
                                _platform.Log($"- Loading {s.Value}");
                                status = $"Fetching app data tag: {s.Value}";
                                XElement ret = XDocument.Load(new Uri(s.Value).AbsoluteUri)
                                    .Element("app");
                                return ret;
                            }))
                        .ToArray();
                    status = $"Creating app cache";
                    foreach (XElement app in tmpApparray)
                    {
                        //"Sanity check"
                        Version.Parse(app.Element("Version").Value);
                        Guid.Parse(app.Element("ID").Value);
                        //Create XElement
                        tmpAppsList.Add(new XElement("App",
                            new XElement("Name", app.Element("Name").Value),
                            new XElement("Description", app.Element("Description").Value),
                            new XElement("Version", app.Element("Version").Value),
                            new XElement("ID", app.Element("ID").Value),
                            new XElement("File", new Uri(app.Element("File").Value).ToString()),
                            new XElement("Hash", app.Element("Hash").Value)
                        ));
                        if (app.Element("MainFile") != null)
                            tmpAppsList.Last().Add(new XElement("MainFile", app.Element("MainFile").Value));
                        if (app.Element("Icon") != null)
                            try
                            {
                                tmpAppsList.Last()
                                    .Add(new XElement("Icon",
                                        _platform.FetchImageB64(new Uri(app.Element("Icon").Value))));
                            }
                            catch
                            {
                                // ignored
                            }
                        tmpAppsList.Last().Add(new XElement("Platform",
                            app.Element("Platform") == null ||
                            !new[] {PlatformCheck.Posix, PlatformCheck.Windows}.Contains(app.Element("Platform")
                                .Value)
                                ? PlatformCheck.CurrentPlatform
                                : app.Element("Platform").Value));
                        XElement app1 = app;
                        if (tmpAppsList.Count(a => a.Element("ID").Value == app1.Element("ID").Value) > 1)
                            tmpAppsList.Where(a => a.Element("ID").Value == app.Element("ID").Value).Reverse()
                                .Skip(1)
                                .ToList().ForEach(a => tmpAppsList.Remove(a));
                    }
#if !DEBUG
                }
                catch (Exception e)
                {
                    _platform.OkDialog(
                        $"Failed to load repo: {repArr[i]}{Environment.NewLine}Last status was: {status}{Environment.NewLine}{e}");
                }
#endif
                i++;
            }
            tmpAppsList.Sort((x, y) =>
                string.Compare(x.Element("Name").Value, y.Element("Name").Value, StringComparison.Ordinal));
            if (meta.Element("LocalRepo") == null)
                meta.Add(new XElement("LocalRepo"));
            XElement repos = meta.Element("LocalRepo");
            repos.RemoveNodes();
            tmpAppsList.ForEach(app => repos.Add(app));
            meta.Save(_pathTool.InfoXml);
        }

        private Version GetVer(XElement el) =>
            int.TryParse(el.Value, out int i) ? new Version(0, 0, 0, i) : Version.Parse(el.Value);

        /// <summary>
        ///     Load the repository cache
        /// </summary>
        /// <param name="errorHandler">Function to call on an exception, will ask the user whether he wants to quit</param>
        public void GetReposFromDisk()
        {
            _apps.Clear();
            string xml = _pathTool.InfoXml;
            XDocument.Load(xml).Element("meta").Element("LocalRepo").Elements().ToList().ForEach(app =>
            {
                if (app.Element("Platform").Value != PlatformCheck.CurrentPlatform) return;
                Guid id = Guid.Parse(app.Element("ID").Value);
                string locInPath = _pathTool.GetInfoPath(id);
                XElement locIn = File.Exists(locInPath) ? XDocument.Load(locInPath).Element("app") : app;
                if (int.TryParse(app.Element("Version").Value, out _))
                    app.Element("Version").Value = UpToolLibV1.MinimumVer.ToString();
                _apps.Add(id, _appFactory.CreateApp(
                    locIn.Element("Name").Value,
                    locIn.Element("Description").Value,
                    Version.Parse(app.Element("Version").Value),
                    app.Element("File").Value,
                    false,
                    app.Element("Hash").Value,
                    id,
                    Color.White,
                    app.Element("Icon") == null
                        ? _platform.GetDefaultIcon()
                        : _platform.ImageFromB64(app.Element("Icon").Value),
                    locIn.Element("MainFile") != null || app.Element("MainFile") != null,
                    locIn.Element("MainFile") == null
                        ? app.Element("MainFile") == null ? "" : app.Element("MainFile").Value
                        : locIn.Element("MainFile").Value
                ));
            });
            Directory.GetDirectories(_pathTool.AppsPath)
                .Where(s => Guid.TryParse(Path.GetFileName(s), out Guid guid) &&
                            !_apps.ContainsKey(guid)).ToList().ForEach(s =>
                {
                    Guid tmp = Guid.Parse(Path.GetFileName(s));
                    try
                    {
                        XElement data = XDocument.Load(_pathTool.GetInfoPath(tmp)).Element("app");
                        _apps.Add(tmp,
                            _appFactory.CreateApp($"(local) {data.Element("Name").Value}", data.Element("Description").Value,
                                UpToolLibV1.MinimumVer, "", true, "", tmp, Color.Red,
                                _platform.GetDefaultIcon(),
                                data.Element("MainFile") != null,
                                data.Element("MainFile") == null ? "" : data.Element("MainFile").Value));
                    }
                    catch (Exception e)
                    {
                        if (_platform.YesNoDialog(
                            $@"An error occured while loading this local repo:
{e.Message}
Do you want to exit? Otherwise the folder will be deleted, possibly causeing problems later.", false))
                            Environment.Exit(0);
                        else
                            Directory.Delete(_pathTool.GetAppPath(tmp), true);
                    }
                });
        }
    }
}